#!/usr/bin/python3
"""
    Asterisk AGI Speech Transcription Application
    This script performs speech transcription in a loop by playing the transcribed text back to the caller.
    * Revision: 1
    * Date: Apr 28, 2021
    * Vendor: Universal Speech Solutions LLC
"""

import sys
from asterisk.agi import *

class SpeechTranscriptionApp:

    """A class representing speech transcription application"""

    def __init__(self, options):
        """Constructor"""
        self.options = options
        self.prompt = "Welcome to speech transcription application. Please speak."
        self.instance = None

        self.status = None
        self.cause = None

    def transcribe_speech(self):
        """Performs a streaming speech transcription"""
        self.grammars = "%s,%s" % (self.compose_speech_grammar(), self.compose_dtmf_grammar())
        self.synth_and_recog()

    def synth_and_recog(self):
        """This is an internal function which calls SynthAndRecog"""
        if not self.prompt:
            self.prompt = ' '
        args = "\\\"%s\\\",\\\"%s\\\",%s" % (
            self.prompt, self.grammars, self.options)
        agi.set_variable('RECOG_STATUS', '')
        agi.set_variable('RECOG_COMPLETION_CAUSE', '')
        self.action = None
        agi.appexec('SynthandRecog', args)
        self.status = agi.get_variable('RECOG_STATUS')
        agi.verbose('got status %s' % self.status)
        if self.status == 'OK':
            self.cause = agi.get_variable('RECOG_COMPLETION_CAUSE')
            agi.verbose('got completion cause %s' % self.cause)
        else:
            agi.verbose('recognition completed abnormally')

    def compose_speech_grammar(self):
        """Composes a built-in speech grammar"""
        grammar = 'builtin:speech/transcribe'
        return grammar

    def compose_dtmf_grammar(self):
        """Composes a built-in DTMF grammar"""
        grammar = 'builtin:dtmf/digits'
        return grammar

    def append_grammar_parameter(self, grammar, name, value, separator):
        """Appends a name/value parameter to the specified grammar"""
        grammar += "%s%s=%s" % (separator, name, value)
        return grammar

    def get_prompt(self):
        """Composes the next prompt"""
        prompt = 'You said ' + self.instance + '. Please speak.'
        agi.verbose('got prompt %s' % prompt)
        return prompt

    def check_dialog_completion(self):
        """Checks wtether the dialog is complete"""
        complete = False
        if self.instance == 'Exit' or self.instance == 'Quit' or self.instance == 'exit' or self.instance == 'quit' or self.instance == '0':
            complete = True
        return complete

    def run(self):
        """Interacts with the caller in a loop until the dialog is complete"""

        processing = True
        while processing:

            self.transcribe_speech()
            processing = True
            if self.status == 'OK':
                if self.cause == '000':
                    self.instance = agi.get_variable('RECOG_INSTANCE(0/0)')
                    agi.verbose("got instance %s" % self.instance)
                    self.prompt = self.get_prompt()
                    if self.check_dialog_completion():
                        self.prompt = 'Thank you. See you next time.'
                        processing = False

                elif self.cause != '001' and self.cause != '002':
                    processing = False
            elif self.cause != '001' and self.cause != '002':
                processing = False

        agi.appexec('MRCPSynth', "\\\"%s\\\"" % self.prompt)

agi = AGI()
options = 'plt=1&b=1&sct=1500&sint=15000&nit=10000'
transcribe_app = SpeechTranscriptionApp(options)

transcribe_app.run()
agi.verbose('exiting')
